%{

package com.cryptware.apduparser;

import java.lang.Math;
import java.io.*;
import java.util.*;
import java.math.BigInteger;
%}

/* YACC Declarations */
%token NUM
%token ALFA
%token ANY
%token CMD
%token ECHO
%token SET
%token RST
%token ALG
%token FUN
%token INCLUDE
%token REM

/* Grammar follows */

%%
input: /* empty string */
 | input line
 {
 	$$ = $1 + $2;

 	System.out.println("line: " + $2);
 }
 ;

line: '\n'
	{
		$$ = "\n";
		//linecounter++;
	}
 	| cmd '\n'
 	{
 		//linecounter++;
 		System.out.println(" " + $1 + " ");
 		$$ = $1 + "\n";

 		_cmds.append("; line no. " + linecounter + "\n");
 		_cmds.append($1 + "\n");

		if(_cmdVect != null)
 			_cmdVect.addElement($1);
 	};

cmd:
	RST
  	{
 		System.out.println(" " + $1 + " ");
 		$$ = "(RST)";
  	}
  	| ECHO text
  	{
 		System.out.println(" " + $1 + " " + $2);
 		$$ = "(EVAL ' \"" + $2 + "\")";
  	}
  	| CMD command
  	{
 		System.out.println(" " + $1 + " " + $2);
 		$$ = "(CMD " + $2 + ")";
  	}
  	| CMD command var
  	{
 		System.out.println(" " + $1 + " " + $2);
 		$$ = "(SETQ " + $3 + " (CMD " + $2 + "))";
  	}
  	| SET hex var
  	{
 		System.out.println(" " + $1 + " " + $2 + " " + $3);
 		$$ = "(SETQ " + $3 + " 'h" + $2 + ")";
  	}
  	| INCLUDE ALFA
  	{
 		$$ = "(LOAD \"" + $2 + "\")";
  	}
  	| ALG params var
  	{
 		System.out.println(" " + $1 + " " + $2 + " " + $3);
  	}
  	| FUN params
  	{
 		System.out.println(" " + $1 + " " + $2);
  	}
  	| FUN params var
  	{
 		System.out.println(" " + $1 + " " + $2 + " " + $3);
  	}
  	| REM ANY
  	{
  	    $$ = "(REM \"" + $2 + "\")";
  		System.out.println($2);
  	};

text:
	any
	{
 		System.out.println(" " + $1);
 		$$ = $1;
  	}
  	| any text
	{
 		System.out.println(" " + $1);
 		$$ = $1 + " " + $2;
  	};

any: ALFA
	{
 		System.out.println(" " + $1);
 		$$ = $1;
  	}
  	| NUM
  	{
  		$$ = $1;
  	}
  	| ANY
  	{
  		$$ = $1;
  	};

var:  '<' ALFA '>'
	{
		$$ = $2;
	};

params: hex ';' params
	{
		$$ = "h" + $1 + " h" + $3;
	};

command: hex
	{
		System.out.println(" " + $1);
		$$ = "'h" + $1 + " NIL NIL";
	}
	| hex resp
	{
		$$ = "'h" + $1 + " " + $2 + " NIL";
	}
	| hex sw1sw2
	{
		$$ = "'h" + $1 + " NIL " + $2;
	}
	| hex resp sw1sw2
	{
		$$ = "'h" + $1 + " " + $2 + " " + $3;
	};

resp:
	'[' hex ']'
	{
		$$ = "'h" + $2;
	};


sw1sw2:
	 '(' hex ')'
	{
		$$ = " 'h" + $2;
	};

hex: NUM
	{
		System.out.println(" " + $1);
		$$ = $1;
	}
	| hex NUM
	{
		System.out.println(" " + $1 + " " + $2);
		$$ = $1 + $2;
	};
%%

void yyerror(String s)
{
 	ps.println("Syntax error at line " + st.lineno());
}

private StreamTokenizer st;
private static Hashtable<String, Integer> _cmdTbl;
private PrintStream ps;

static
{
	_cmdTbl = new Hashtable<String, Integer>();
	_cmdTbl.put("cmd", new Integer(CMD));
    _cmdTbl.put("rst", new Integer(RST));
    _cmdTbl.put("rem", new Integer(REM));
    _cmdTbl.put("echo", new Integer(ECHO));
    _cmdTbl.put("set", new Integer(SET));
    _cmdTbl.put("alg", new Integer(ALG));
    _cmdTbl.put("fun", new Integer(FUN));
    _cmdTbl.put("include", new Integer(INCLUDE));
}

private boolean newline;
private StringBuffer _cmds = new StringBuffer();
private int linecounter;
private Vector _cmdVect;
boolean bREM = false;
boolean bEOL = true;
boolean bNeedEOL = false;
String  strString;
private int yylex()
{
	int tok;

 	try
 	{
 	   if(bNeedEOL)
 	   {
 	       bNeedEOL = false;
 	       return '\n';
 	   }

 	   tok = st.nextToken();

 	   if(tok == st.TT_EOF)
 	   {
 	   	   bEOL = false;
 	       if(newline)
 	       {
 	           bREM = false;
 	           return 0;
 	       }
 	       else
 	       {
 	          if(bREM)
 	          {
 	          		linecounter++;
 	                bREM = false;
 	                st.sval = strString;
 	                yylval = strString;
 	                return ANY;
 	          }
 	          else
 	          {
 	              newline = true;
 	              return '\n';
 	          }
 	       }

 	   }
 	   else if(tok == st.TT_EOL)
       {
 	      linecounter++;
 	      bEOL = true;
 	      if(bREM)
 	      {
 	      		bREM = false;
 	      		st.sval = strString;
 	      		yylval = strString;
 	      		bNeedEOL = true;
 	      		return ANY;
 	      }
       }
       else if(bREM)
       {
       		strString += (st.sval == null ? "" : st.sval) + " ";
       		return yylex();
       }
       else if(tok == '\\')
       {
       		bEOL = false;
       		if(st.nextToken() == '\n')
       		{
       			return yylex();
       		}
       		else
       		{
       			tok = ANY;
 	   			yylval = st.sval;
 	   		}
       }
 	   else if(tok == st.TT_WORD)
 	   {
 	       yylval = st.sval; //SEE BELOW
 	       if(bEOL && yylval.toLowerCase().equals("rem"))
 	       {
 	       		bREM = true;
 	       		strString = "";
 	       		return REM;
 	       }

 	       bEOL = false;

 	       try
 	       {
 	           BigInteger hex = new BigInteger(st.sval, 16);
 	           tok = NUM;
 	       }
 	       catch(Exception ex)
 	       {
 	           //ex.printStackTrace();
 	           if(_cmdTbl.containsKey(st.sval))
 	           {
 	              tok = _cmdTbl.get(st.sval).intValue();
 	           }
 	           else
 	           {
 	              tok = ALFA;
 	           }
 	       }
 	   }
 	   else if("'()[];<>\n\r\"".indexOf(tok) < 0)
 	   {
 	   		tok = ANY;
 	   		yylval = st.sval;
 	   }
 	}
 	catch (Exception e)
 	{
 		return -1;
 	}
 	return tok;
}

void dotest()
{
    //BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
    BufferedReader in = new BufferedReader(new StringReader("RST \n CMD 00 C0 \\ \n 01 83 4F \n echo ugo chirico \\ \n ornellina \nREM SET 01 02 0304 <UGO>"));

    System.out.println("BYACC/J Calculator Demo");
    System.out.println("Note: Since this example uses the StringTokenizer");
    System.out.println("for simplicity, you will need to separate the items");
    System.out.println("with spaces, i.e.: '( 3 + 5 ) * 2'");

    System.out.println("result: " + parse(in, System.out));
}


public String parse(Reader in, PrintStream out, Vector cmdVect)
{
    _cmdVect = cmdVect;
    return parse(in, out);
}

public String parse(Reader in, PrintStream out)
{
	ps = out;
    st = new StreamTokenizer(in);

    st.commentChar('/');
    st.commentChar('\'');
	st.quoteChar('"');
    st.lowerCaseMode(true);
    st.ordinaryChars('0', '9');
    st.wordChars('0', '9');
    st.wordChars('A', 'Z');
    st.wordChars('a', 'z');
 	st.eolIsSignificant(true);
    //st.whitespaceChars(0, ' ');
    newline=false;
	linecounter = 0;
	if(yyparse() == 0) // OK
		return _cmds.toString();
	else
		return null;
}

public static void main(String args[])
{
 Parser par = new Parser(false);
 par.dotest();
}