package com.ugos.JILisp.engine;

%%

%{

String m_str;
String m_quote;

void print(String str)
{
    //System.out.println("Read: " + str);
}

void print()
{
    //System.out.println("Read: " + yytext());
}

%}

%line
%char
%class JILLex
%state COMMENT_SEMICOLON
%state STRING
%notunix

%eofval{
    return (new Yytoken(parser.TOK_END,yytext(),yyline,yychar,yychar+1));
%eofval}

ALPHA=["*"0-9A-Za-z_-]
DIGIT=[0-9]
WHITE_SPACE_CHAR=[\n\r\ \t\b\012]

MATH_OP=">"|"<"|">="|"<="|"="|"=\="
ATOM=({DIGIT}*)({ALPHA}+)({DIGIT}*)
SYMBOL={ATOM}|("&"{ATOM})|(":"{ATOM})

FLOAT=({DIGIT}+)["."]({DIGIT}+)["e"]({DIGIT}+)|({DIGIT}+)["."]({DIGIT}+)["e"]["-"]({DIGIT}+)
NUMBER=({DIGIT}+)|({DIGIT}+)["."]({DIGIT}+)|{FLOAT}
RATIONAL_NUMBER=({DIGIT}+)["/"]({DIGIT}+)

%%


<COMMENT_SEMICOLON> [\r\n] {yybegin(YYINITIAL);}
<COMMENT_SEMICOLON> . {print();}

<STRING> \"
{
    if(m_quote.equals(yytext()))
    {
        yybegin(YYINITIAL);
        System.out.println("String");
        return (new Yytoken(parser.TOK_STRING,m_str,yyline,yychar,yychar + m_str.length()));
    }
    else
    {
        m_str += yytext();
    }
}

<STRING> \\\"
{
    print();
    m_str += "\"";
}

<STRING> \\\\
{
    print();
    m_str += "\\";
}

<STRING> [\n\r] {}

<STRING> .
{
    print();
    m_str += yytext();
}


<YYINITIAL> \"
{
    print();
    m_str = "";
    m_quote = yytext();
    yybegin(STRING);
}

<YYINITIAL> ";"
{
    print(";");
    yybegin(COMMENT_SEMICOLON);
}

<YYINITIAL> {NUMBER}|-{NUMBER}|["+"]{NUMBER}
{
    print();
    String strTok =  yytext();
    return (new Yytoken(parser.TOK_NUM,strTok,yyline,yychar,yychar + strTok.length()));
}

<YYINITIAL> {RATIONAL_NUMBER}|-{RATIONAL_NUMBER}|["+"]{RATIONAL_NUMBER}
{
    print();
    String strTok =  yytext();
    return (new Yytoken(parser.TOK_RATNUM,strTok,yyline,yychar,yychar + strTok.length()));
}

<YYINITIAL> {SYMBOL}|{SYMBOL}-{SYMBOL}
{
    print();
    String strTok =  yytext();
    return (new Yytoken(parser.TOK_SYMBOL,strTok,yyline,yychar,yychar + strTok.length()));
}

<YYINITIAL> "#C" {print(); return (new Yytoken(parser.TOK_COMPLEX,"function",yyline,yychar,yychar+1)); }
<YYINITIAL> "#c" {print(); return (new Yytoken(parser.TOK_COMPLEX,"function",yyline,yychar,yychar+1)); }
<YYINITIAL> "#" {print(); return (new Yytoken(parser.TOK_SHARP,"function",yyline,yychar,yychar+1)); }
<YYINITIAL> "'" {print(); return (new Yytoken(parser.TOK_QUOTE,"quote",yyline,yychar,yychar+1)); }
<YYINITIAL> "." {print(); return (new Yytoken(parser.TOK_DOT,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ">" {print(); return (new Yytoken(parser.TOK_SYMBOL,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "<" {print(); return (new Yytoken(parser.TOK_SYMBOL,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ">=" {print(); return (new Yytoken(parser.TOK_SYMBOL,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "<=" {print(); return (new Yytoken(parser.TOK_SYMBOL,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "=" {print(); return (new Yytoken(parser.TOK_SYMBOL,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "+" {print(); return (new Yytoken(parser.TOK_SYMBOL,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "/" {print(); return (new Yytoken(parser.TOK_SYMBOL,yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> "(" {print(); return (new Yytoken('(',yytext(),yyline,yychar,yychar+1)); }
<YYINITIAL> ")" {print(); return (new Yytoken(')',yytext(),yyline,yychar,yychar+1)); }

<YYINITIAL> {WHITE_SPACE_CHAR}
{
    // Do nothing
}

.
{
    //System.out.println("dot");
    print();
    return (new Yytoken(yytext().charAt(0),yytext(),yyline,yychar,yychar+1));
//  print();
//  throw new RuntimeException("Unrecognized character at line "+ yyline);
}











