%{
package com.ugos.JILisp.engine;

import java.io.*;
import java.util.*;
%}

/* YACC Declarations */
%token TOK_SYMBOL
%token TOK_SHARP
%token TOK_QUOTE
%token TOK_NUM
%token TOK_RATNUM
%token TOK_COMPLEX
%token TOK_DOT
%token TOK_END
%token TOK_STRING
%left '(' ')'


/* Grammar follows */
%%

/////////////////////
// Start
/////////////////////
%start s_expression_list

////////////////////////////
// Definizione di s_expression
////////////////////////////

s_expression: TOK_SYMBOL
              {
                //if (((String)$1).toUpperCase().equals("NIL"))
                //  $$ = new JILCons(JILSymbol.NIL, JILSymbol.NIL).getCell();
                //else
                    $$ = new LispCell(JILSymbol.create((String)$1));
                //System.out.println(((JILExpression)$$).prettyPrint());
              }
            | number
              {
                 $$ = new LispCell((JILNumber)$1);
              }
            | complex
             {
                $$ = new LispCell((JILComplex)$1);;
             }
            | TOK_STRING
              {
                  $$ = new LispCell(new JILString((String)$1));
                  //System.out.println(((JILExpression)$$).prettyPrint());
              }
            | cons
              {
                  $$ = $1;
                  //System.out.println(((JILExpression)$$).prettyPrint());
              }
            | TOK_QUOTE s_expression
              {
                  $$ = new LispCell(
                            new JILCons(
                                new LispCell(JILSymbol.create("quote")),
                                new LispCell(
                                        new JILCons((LispCell)$2, JILSymbol.NIL.getCell()))));
              }
            | TOK_SHARP TOK_QUOTE s_expression
              {
                  $$ = new LispCell(
                            new JILCons(
                                new LispCell(JILSymbol.create("function")),
                                new LispCell(
                                        new JILCons((LispCell)$3, JILSymbol.NIL.getCell()))));
              };

cons:   dotted_pair
        {
            $$ = $1;
            //System.out.println(((JILExpression)$$).prettyPrint());
        }
     |  list
        {
            $$ = $1;
            //System.out.println(((JILExpression)$$).prettyPrint());
        };

dotted_pair: '(' s_expression TOK_DOT s_expression ')'
             {
                $$ = new LispCell(new JILCons((LispCell)$2, (LispCell)$4));
                //System.out.println(((LispExpression)$$).prettyPrint());
             };
             
list: '(' s_expression_list ')'
      {
            $$ = $2;
            //System.out.println(((LispExpression)$$).prettyPrint());
      }
     | '(' ')'
      {
            //$$ = null;
            //$$ = new LispCons(LispSymbol.NIL, LispSymbol.NIL).getCell();
            $$ = JILSymbol.NIL.getCell();
      };
      
s_expression_list: s_expression
           {
                $$ = new LispCell(new JILCons((LispCell)$1, JILSymbol.NIL.getCell()));
                //System.out.println(((LispExpression)$$).prettyPrint());
           }
        |  s_expression s_expression_list
           {
                $$ = new LispCell(new JILCons((LispCell)$1, (LispCell)$2));
                //System.out.println(((LispExpression)$$).prettyPrint());
           };
           
number:     TOK_NUM
              {
                $$ = new JILNumber(((Double)$1).doubleValue());
                //System.out.println(((JILExpression)$$).prettyPrint());
              }
            | TOK_RATNUM
              {
                $$ = new JILRationalNumber((String)$1);
                //System.out.println(((JILExpression)$$).prettyPrint());
            };
            
complex:  TOK_COMPLEX '(' number number ')'
            {
                $$ = new JILComplex((JILNumber)$3, (JILNumber)$4);
            };
            
// End grammar rules
%%

Yytoken m_lastToken = null;
JILLex m_lex;
Reader is;
JILExpression m_sexp;
//JILEngine m_engine;

String    m_strLastClWord    = "";

boolean   m_bPrompt          = false;
String    m_strError         = "";
String    m_strErrorPlus     = "";

public parser(boolean debug,  Reader ins)
{
    this(debug);
    
    m_strError = "";
    is = ins;
    initLex();
}

public LispCell parse() throws JILSyntaxErrorException
{
    if(yyparse() == 0)
    {
        return (LispCell)yyval;
    }
    else
    {
        throw new JILSyntaxErrorException("Syntax Error at " + m_strError);
        //return LispSymbol.NIL.getCell();
    }
}

String getError()
{
    return m_strError;
}

void println(String st)
{
    System.out.println(st);
}

void reduce(String st)
{
    System.out.println("Reduce by " + st);
}

void initLex()
{
    m_lex = new JILLex(is);
}

void error(String s)
{
    m_strErrorPlus = s;
    yyerror("");
}

void yyerror(String s)
{
    //System.out.println("Error:" + s);
    //System.out.println("at line:" + Integer.toString(st.lineno()));
    //m_strError = s + " at line " + Integer.toString(st.lineno());
        
    if(m_lastToken != null)
        m_strError = "Line " + Integer.toString(m_lastToken.m_line + 1) + " near to " + m_lastToken.m_strToken + ((m_strErrorPlus != "") ? (" : " + m_strErrorPlus) : (""));
}

int yylex()
{
    Yytoken token = null;
    try
    {
        token = m_lex.yylex();
    }
    catch(IOException ex)
    {
        ex.printStackTrace();
        error(ex.getMessage());
        throw new RuntimeException("IO Exception : " + ex.getMessage());
        //return -1;
    }
    catch(Error er)
    {
        er.printStackTrace();
        error(er.getMessage());
        throw new RuntimeException("Syntax Error : " + er.getMessage());
        //return -1;
    }

    if (token == null)
    {
        return -1;
    }
    else
    {
        if (token.m_nToken == TOK_END)
        {
            return -1;
        }
        else if (token.m_nToken == TOK_NUM)
        {
            try
            {
                yylval = Double.valueOf(token.m_strToken);
            }
            catch (NumberFormatException ex)
            {
                ex.printStackTrace();
                error(ex.getMessage());
                throw new RuntimeException("Number Exception : " + ex.getMessage());
                //return -1;
            }
        }
        else
        {
            yylval = token.m_strToken;
        }

        m_lastToken = token;
        return token.m_nToken;
    }
}


